package main

import (
	hconsole "git.swzry.com/zry/GoHiedaLogger/hiedabke_console"
	"git.swzry.com/zry/GoHiedaLogger/hiedalog"
	lazyquiter "git.swzry.com/zry/go-lazy-quiter"
	rpcore "git.swzry.com/zry/ran-proc/rpcore"
	wslogdist "git.swzry.com/zry/ran-proc/wslogdist"
	"github.com/hashicorp/go-uuid"
	"github.com/oklog/run"
	"os"
)

var Logger *hiedalog.HiedaLogger
var PM *rpcore.ChildProcManager
var WebServer *WebServerClass
var AccessKey string
var WsLogDistr *wslogdist.WsLogDistributor
var LazyQuiter *lazyquiter.RunGroupQuiter
var GracefulShutdownActorForThis rpcore.GracefulShutdownActor

func main() {
	Logger = hiedalog.NewHiedaLogger()
	Logger.AddBackend(hconsole.NewConsoleBackend(os.Stdout), Logger.LevelFilter.NameToID(hiedalog.DLN_DEBUG))
	AccessKey, _ = uuid.GenerateUUID()
	Logger.LogPrint("key", hiedalog.DLN_INFO, "Access Key:", AccessKey)
	LazyQuiter = lazyquiter.NewRunGroupQuiter()
	LazyQuiter.SignalNotifyFunc = func(sig os.Signal) {
		Logger.LogPrint("main", hiedalog.DLN_INFO, "Quit by user abort.")
	}
	GracefulShutdownActorForThis = &rpcore.UnixSignalShutdownActor{}
	PM = rpcore.NewChildProcManager(&rpcore.DefaultChildProcManagerMsgHandler{
		Logger:        Logger,
		LogModuleName: "pm",
	})
	WsLogDistr = wslogdist.NewWsLogDistributor(1024)
	WebServer = NewWebServer()
	rg := &run.Group{}
	rg.Add(PMRun, PMStop)
	rg.Add(WebServer.Run, WebServer.Stop)
	rg.Add(LazyQuiter.Run, LazyQuiter.Stop)
	err := rg.Run()
	if err != nil {
		Logger.LogPrint("main", hiedalog.DLN_FATAL, "error in running:", err)
	}
	os.Exit(0)
}
