package rpcore

import (
	"git.swzry.com/zry/GoHiedaLogger/hiedalog"
)

type ChildProcManagerMsgHandler interface {
	ManagerStart()
	ManagerStop(err error)
	MsgLoopVerboseLog(CPID int64, text string)
	ProcessStarted(CPID int64)
	ProcessQuit(CPID int64, err error)
}

type DefaultChildProcManagerMsgHandler struct {
	Logger        *hiedalog.HiedaLogger
	LogModuleName string
}

func (d DefaultChildProcManagerMsgHandler) ManagerStart() {
	if d.Logger == nil {
		return
	}
	d.Logger.LogPrint(d.LogModuleName, hiedalog.DLN_INFO, "<procman> start running")
}

func (d DefaultChildProcManagerMsgHandler) ManagerStop(err error) {
	if d.Logger == nil {
		return
	}
	if err != nil {
		d.Logger.LogPrint(d.LogModuleName, hiedalog.DLN_INFO, "<procman> stopped with error: ", err)
	} else {
		d.Logger.LogPrint(d.LogModuleName, hiedalog.DLN_INFO, "<procman> stopped normally")
	}
}

func (d DefaultChildProcManagerMsgHandler) MsgLoopVerboseLog(CPID int64, text string) {
	if d.Logger == nil {
		return
	}
	d.Logger.LogPrintf(d.LogModuleName, hiedalog.DLN_VERBOSE, "<msgloop> | %16x | %s", CPID, text)
}

func (d DefaultChildProcManagerMsgHandler) ProcessQuit(CPID int64, err error) {
	if err == nil {
		d.Logger.LogPrintf(d.LogModuleName, hiedalog.DLN_INFO, "<procend> | %16x | end normally", CPID)
	} else {
		d.Logger.LogPrintf(d.LogModuleName, hiedalog.DLN_WARN, "<procend> | %16x | end with error: %v", CPID, err)
	}
}

func (d DefaultChildProcManagerMsgHandler) ProcessStarted(CPID int64) {
	d.Logger.LogPrintf(d.LogModuleName, hiedalog.DLN_INFO, "<procend> | %16x | started", CPID)
}
